package {	

	import flash.display.BitmapData;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.media.Sound;
	import flash.text.TextField;
	
	// https://www.greensock.com/tweenlite/
	import com.greensock.TweenLite;

	public class Main extends Sprite {
		
		private static const MOVE_TIME:Number = 0.7;
		private static const MOVE_DELAY:Number = 0.2;
		private static const LEVEL_LABELS:Array = ["one", 	"two", 	"three", 	"four", "five", "six", "seven", "eight", "nine", "ten"];
		private static const LEVEL_TIMES:Array = [15, 		15,		 30, 		25, 	30, 	30, 	50, 	60, 	60, 	75];
		private static const TIME_PENALTY:int = 2;
		
		public var wayfinder:Sprite;
		public var fader:Sprite;
		public var clock:Clock;
		public var t_retry:TextField;
		
		private var _currentRoom:Room;
		private var _oldRoom:Room;
		private var _levelNum:int;
		
		private var _currentPos:Point;
		private var _sounds:Array;
		private var _map:Map;
		private var _sadFace:Sprite;
		private var _numTries:int = 0;
		
		private var _appearSound = new AppearSound();
		
		// CONSTRUCTOR
		public function Main() {
			super();
			
			_sadFace = new SadFace();
			_sadFace.x = 400;
			_sadFace.y = 300;
			
			createRooms();
			
			_levelNum = 0;
			
			_map = new Map();
			_map.loadLevel(_levelNum);
			_sounds = [new Sound1(), new Sound2(), new Sound3(), new Sound4(), new Sound5()];
			
			clock.addEventListener(Clock.TIME_UP, onTimeUp);
			
			startGame();
			wayfinder.visible = false;
		}
		
		// ---------------------------------------------------------------
		// ::: PRIVATE ::: 
		// ---------------------------------------------------------------
		
		//-----------------------------------------------------------------------------
		protected function createRooms():void {
			_currentRoom = new Room();
			for each(var b:MovieClip in _currentRoom.buttons){
				b.addEventListener(MouseEvent.CLICK, onButtonClick);
			}
			
			_oldRoom = new Room();
			for each(b in _oldRoom.buttons){
				b.addEventListener(MouseEvent.CLICK, onButtonClick);
			}
		}
		
		//-----------------------------------------------------------------------------
		protected function onButtonClick(e:MouseEvent):void {
			var b:MovieClip = e.currentTarget as MovieClip;
			var dir:int = _currentRoom.buttons.indexOf(b);
			
			tryToMoveWithDir(dir);
		}
		
		//-----------------------------------------------------------------------------
		protected function startGame():void {
			_currentRoom.setMapImage(_map.mapImage);
			_oldRoom.setMapImage(_map.mapImage);
			
			if(_numTries > 0){
				showRetryText();
			}
			
			
			var time:int = LEVEL_TIMES[_levelNum] - (_numTries * TIME_PENALTY);
			clock.setTime(time);
			
			addChild(_currentRoom);
			addChild(fader);
			_currentPos = _map.startPos.clone();
			configureRoom(_currentRoom);
			_currentRoom.hideButtons();
			
			updateWayfinder();
			TweenLite.to(fader, 1.0, {alpha: 0, onComplete: onFadeInComplete});
		}
		
		//-----------------------------------------------------------------------------
		protected function showRetryText():void {
			t_retry.x = 225;
			t_retry.y = -10;
			t_retry.alpha = 1;
			
			TweenLite.to(t_retry, 0.5, {y: 260});
		}
		
		//-----------------------------------------------------------------------------
		protected function onFadeInComplete():void {
			removeChild(fader);
			_currentRoom.fadeInButtons();
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			
			_appearSound.play();
			clock.start();
			
			if(_numTries > 0){
				TweenLite.to(t_retry, 1, {alpha: 0});
			}
		}
		
		//-----------------------------------------------------------------------------
		protected function onKeyDown(e:KeyboardEvent):void {
			if(isArrowKey(e.keyCode)){
				var dir:int = Direction.fromKeyCode(e.keyCode)
				tryToMoveWithDir(dir);
			}
		}
		
		//-----------------------------------------------------------------------------
		protected function tryToMoveWithDir(dir:int):void {
			var canMove:Boolean = _currentRoom.move(dir);
			if(canMove){
				stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
				_sounds[0].play();
				movePosition(dir);
				moveRooms(dir);
				updateWayfinder();
			}
		}
		
		//-----------------------------------------------------------------------------
		protected function updateWayfinder():void {
			var dy:Number = _map.goalPos.y - _currentPos.y;
			var dx:Number = _map.goalPos.x - _currentPos.x;
			
			var a:Number = Math.atan2(dy, dx) * 180 / Math.PI;
			a += 90;
			
			if(a > 180) { 
				a = 180 - a;
			}
			
			TweenLite.to(wayfinder, 0.25, {rotation: a});
		}
		
		//-----------------------------------------------------------------------------
		protected function movePosition(dir):void {
			switch(dir)	{
				case Direction.UP:
					_currentPos.y --;
				break;
				
				case Direction.DOWN:
					_currentPos.y ++;
				break;
				
				case Direction.RIGHT:
					_currentPos.x ++;
				break;
				
				case Direction.LEFT:
					_currentPos.x --;
				break;
			}
		}
		
		//-----------------------------------------------------------------------------
		protected function moveRooms(dir:int):void {
			var newRoom:Room = _oldRoom;
			_oldRoom = _currentRoom;
			_currentRoom = newRoom;
			
			var targetPos:Point = Direction.targetPosForDir(dir);
			TweenLite.to(_oldRoom, MOVE_TIME, {delay:MOVE_DELAY, x:targetPos.x, y:targetPos.y});
			
			_currentRoom.x = targetPos.x * -1;
			_currentRoom.y = targetPos.y * -1;
			configureRoom(_currentRoom);
			addChild(_currentRoom);
			TweenLite.to(_currentRoom, MOVE_TIME, {delay:MOVE_DELAY, x:0, y:0, onComplete:onMoveComplete});
		}
		
		//-----------------------------------------------------------------------------
		protected function configureRoom(room:Room):void {
			
			var markerFrame:String = "";
			
			var roomType:int = _map.getRoomType(_currentPos);
			var levelText:String = "";
			var storyText:String = "";
			if(roomType == Room.START){
				levelText = "level " + LEVEL_LABELS[_levelNum];
			} else if(roomType == Room.STORY){
				storyText = Story.getTextForRoomInLevel(_currentPos, _levelNum);
			}
			
			if(roomType == Room.GOAL){ clock.stop(); }
			room.configure(_map.getDirsForRoom(_currentPos), roomType, levelText, storyText);
		}
		
		//-----------------------------------------------------------------------------
		protected function onMoveComplete():void {
			removeChild(_oldRoom);
			
			if(_map.isGoal(_currentPos)){
				doLevelComplete();
			} else {
				stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			}
			
		}
		
		//-----------------------------------------------------------------------------
		protected function doLevelComplete():void {
			_sounds[4].play();
			var d:Number = 0.3;
			
			//var snd:Sound = new CompleteSound();
			TweenLite.delayedCall(d, _sounds[2].play);
			
			TweenLite.to(_currentRoom.marker, 0.4, {delay: d, scaleX: 4, scaleY: 4, y: 300});
			addChild(fader);
			TweenLite.to(fader, 0.75, {delay: d, alpha: 1, onComplete: nextLevel});
		}
		
		//-----------------------------------------------------------------------------
		protected function nextLevel():void {
			_levelNum++;
			_numTries = 0;
			
			if(_levelNum < _map.numLevels){
				_map.loadLevel(_levelNum);
				TweenLite.delayedCall(0.5, startGame);
			} else {
				doGameWon();
			}
		}
		
		//-----------------------------------------------------------------------------
		protected function isArrowKey(code:int):Boolean {
			return (code >= 37 && code <=40);
		}
		
		//-----------------------------------------------------------------------------
		protected function onTimeUp(e:Event):void {
			stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			doGameOver();
		}
		
		//-----------------------------------------------------------------------------
		protected function doGameOver():void {
			_sadFace.scaleX = _sadFace.scaleY = 1;
			addChild(_sadFace);
			var snd:Sound = new DieSound();
			snd.play();
			
			var d:Number = 0.3;
			//TweenLite.delayedCall(d, _sounds[1].play);
			
			TweenLite.to(_sadFace, 0.4, {delay: d, scaleX:4, scaleY:4});
			addChild(fader);
			TweenLite.to(fader, 0.75, {delay: d, alpha: 1, onComplete: restartGame});
		}
		
		//-----------------------------------------------------------------------------
		protected function doGameWon():void {
			clock.visible = false;
			clock.stop();
			
			addChild(_currentRoom);
			addChild(fader);
			
			
			_currentRoom.marker.gotoAndStop("win");
			
			var levelText:String = "you win";
			var storyText:String = "";
			
			_currentRoom.configure([], Room.ROOM, levelText, storyText);
			_currentRoom.marker.visible = true;
			_currentRoom.hideButtons();

			TweenLite.to(fader, 1.0, {alpha: 0});
		}
		
		//-----------------------------------------------------------------------------
		protected function restartGame():void {
			removeChild(_sadFace);
			_numTries ++;
			t_retry.text = "RETRY LEVEL";
			
			if(_numTries >= 3 || LEVEL_TIMES[_levelNum] - _numTries * TIME_PENALTY < 10){
				_levelNum--;
				_numTries = 1;
				t_retry.text = "RETRY PREVIOUS LEVEL";
			}
			
			if(_levelNum < 0){
				_levelNum = 0;
				_numTries = 0;
			}
			
			_map.loadLevel(_levelNum);
			TweenLite.delayedCall(0.5, startGame);
		}
		
		
	}

}