package {	

	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.geom.Point;

	public class Map {
		
		
		
		private var _map:Array;
		private var _width:int;
		private var _height:int;
		private var _startPos:Point;
		private var _goalPos:Point;
		private var _mapImgs:Array;
		
		// CONSTRUCTOR
		public function Map() {
			super();
			_mapImgs = [new Map1(0,0), new Map2(0,0), new Map3(0,0), new Map4(0,0), new Map5(0,0), new Map6(0,0), new Map7(0,0), new Map8(0,0), new Map9(0,0), new Map10(0,0)];
		}
		
		public function get numLevels():int {
			return _mapImgs.length;
		}
		
		//-----------------------------------------------------------------------------
		public function loadLevel(levelNum:int):void {
			loadMap(_mapImgs[levelNum]);
		}
		
		//-----------------------------------------------------------------------------
		public function getDirsForRoom(pos:Point):Array {
			var dirs:Array = [];
			
			if(pos.x > 0 && _map[pos.y][pos.x - 1]){
				dirs.push(Direction.LEFT);
			}
			
			if(pos.x < _width-1 && _map[pos.y][pos.x + 1]){
				dirs.push(Direction.RIGHT);
			}
			
			if(pos.y > 0 && _map[pos.y - 1][pos.x]){
				dirs.push(Direction.UP);
			}
			
			if(pos.y < _height-1 && _map[pos.y + 1][pos.x]){
				dirs.push(Direction.DOWN);
			}
			
			return dirs;
		}
		
		public function get startPos():Point {
			return _startPos;
		}
		
		public function get goalPos():Point {
			return _goalPos;
		}
		
		//-----------------------------------------------------------------------------
		public function isGoal(pos:Point):Boolean {
			return (_map[pos.y][pos.x] == Room.GOAL);
		}
		
		//-----------------------------------------------------------------------------
		public function isMapRoom(pos:Point):Boolean {
			return _map[pos.y][pos.x] == Room.MAP;
		}
		
		public function get mapImage():Sprite {
			return drawImage();
		}
		
		//-----------------------------------------------------------------------------
		public function getRoomType(pos:Point):int {
			return _map[pos.y][pos.x];
		}
		
		// ---------------------------------------------------------------
		// ::: PRIVATE ::: 
		// ---------------------------------------------------------------
		
		//-----------------------------------------------------------------------------
		protected function drawImage():Sprite {
			var roomSize:int = 10;
			var roomSpacing:int = 2;
			var markerSize:int = 4;
			
			var map:Sprite = new Sprite();
			for(var row:int = 0; row < _map.length; row++){
				for(var col:int = 0; col < _map[row].length; col++){
					if(_map[row][col] > 0){
						map.graphics.beginFill(0xCCCCCC);
						map.graphics.drawRect(col * roomSize + col * roomSpacing, row * roomSize + row * roomSpacing, roomSize, roomSize);
						if(_map[row][col] == Room.GOAL){
							map.graphics.beginFill(0x00FF66);
							map.graphics.drawRect(col * roomSize + col * roomSpacing + (roomSize - markerSize) / 2, row * roomSize + row * roomSpacing + (roomSize - markerSize)/2, markerSize, markerSize);
						} else if(_map[row][col] == Room.START){
							map.graphics.beginFill(0x00FFFF);
							map.graphics.drawRect(col * roomSize + col * roomSpacing + (roomSize - markerSize) / 2, row * roomSize + row * roomSpacing + (roomSize - markerSize)/2, markerSize, markerSize);
						}
					}
				}
			}
			
			return map;
		}

		//-----------------------------------------------------------------------------
		protected function loadMap(mapImg:BitmapData):void {
			_map = [];
			
			for(var row:int = 0; row < mapImg.height; row++){
				_map[row] = [];
				for(var col:int = 0; col < mapImg.width; col++){
					var px:uint = mapImg.getPixel(col, row);
					
					switch(px){
						case 0:
							_map[row][col] = 0;
						break;
						case 0x00FF00:
							_goalPos = new Point(col, row);
							_map[row][col] = Room.GOAL;
						break;	
						case 0xFF0000:
							_map[row][col] = Room.STORY;//Room.MAP;
						break;
						
						case 0x0000FF:
							_startPos = new Point(col, row);
							_map[row][col] = Room.START;
						break;	
						
						
						default: 
							_map[row][col] = Room.ROOM;
						
					}
				}
			}
			
			_height = _map.length;
			trace(_height);
			_width = _map[0].length;
		}

	}

}