package {	

	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.text.TextField;
	
	import com.greensock.TweenLite;

	public class Room extends Sprite {
		
		public static const ROOM:int = 1;
		public static const START:int = 2;
		public static const GOAL:int = 3;
		public static const MAP:int = 4;
		public static const BEACON:int = 5;
		public static const STORY:int = 6;
		
		// movieclips
		public var b_up:MovieClip;
		public var b_down:MovieClip;
		public var b_right:MovieClip;
		public var b_left:MovieClip;
		
		public var t_levelNum:TextField;
		public var t_story:TextField;
		public var marker:MovieClip;
		
		private var _directions:Array;
		private var _buttons:Array;
		private var _map:Sprite;
		
		// CONSTRUCTOR
		public function Room() {
			super();
			
			_buttons = [];
			_buttons[Direction.UP] = b_up;
			_buttons[Direction.DOWN] = b_down;
			_buttons[Direction.RIGHT] = b_right;
			_buttons[Direction.LEFT] = b_left;
			
			marker.gotoAndStop(1);
		}
		
		public function get buttons():Array {
			return _buttons;
		}

		//-----------------------------------------------------------------------------
		public function setMapImage(map:Sprite):void {
			if(_map){
				removeChild(_map);
				_map = null;
			}
			
			_map = map;
			_map.x = Math.floor(400 - _map.width/2);
			_map.y = Math.floor(170 - _map.height);
			
			addChild(_map);
			
		}

		//-----------------------------------------------------------------------------
		public function hideButtons():void {
			for each(var b:MovieClip in _buttons){
				b.alpha = 0;
			}
		}
		
		//-----------------------------------------------------------------------------
		public function fadeInButtons():void {
			for each(var b:MovieClip in _buttons){
				TweenLite.to(b, 0.5, {alpha: 1.0});
			}
		}
		
		//-----------------------------------------------------------------------------
		public function configure(dirs:Array, roomType:int, levelText:String = "", storyText:String = ""):void {
			reset();
			
			t_levelNum.text = levelText.toUpperCase();
		//	t_story.text = storyText;
			
			switch(roomType){
				case 0:
				break;
				
				case GOAL:
					marker.gotoAndStop("goal");
					marker.visible = true;
				break;
				
				case START:
					marker.gotoAndStop("start");
					marker.visible = true;
					_map.visible = true;
				break;
				
				case MAP:
				//	_map.visible = true;
				//	t_levelNum.text = "MAP";
				break;	
				
			}
			
			
			if(roomType != GOAL){
				for each(var dir:int in dirs){
					_directions[dir] = true;
					_buttons[dir].visible = true;
				}
			}
		}
		
		//-----------------------------------------------------------------------------
		public function move(dir:int):Boolean {
			if(dir >=0 && _directions[dir]){
				_buttons[dir].gotoAndStop("HL");
				return true;
			}
			
			return false;
		}
		
		// ---------------------------------------------------------------
		// ::: PRIVATE ::: 
		// ---------------------------------------------------------------
		
		//-----------------------------------------------------------------------------
		private function reset():void {
			_directions = [];
			for each(var b:MovieClip in _buttons){
				b.gotoAndStop(1);
				b.visible = false;
			}
			
			marker.visible = false;
			marker.scaleX = marker.scaleY = 1;
		//	marker.y = 210;
			
			if(_map) _map.visible = false;
		}

	}

}